import { defHttp } from '~/utils/http'

// 密码登录
export function login(data: API.LoginReq) {
  return defHttp.post<API.LoginResp>({
    url: '/auth/login',
    method: 'POST',
    data,
  }, { withToken: false })
}

// 验证码登录，手机x邮箱
export function loginByCode(data: API.LoginReq) {
  return defHttp.post<API.LoginResp>({
    url: '/auth/login/byCode',
    method: 'POST',
    data,
  }, { withToken: false })
}

// Oauth 第三方登录，获取登录地址（通常针对PC网页登录，APP端自行获取code）
export function oauthLogin(params: API.OauthLoginReq) {
  return defHttp.post<API.OauthLoginResp>({
    url: '/auth/oauth/login',
    method: 'GET',
    params,
  }, { withToken: false })
}

// Oauth 第三方登录，客户端自行获取code进行登录
export function oauthLoginByCode(data: API.OauthLoginByCodeReq) {
  return defHttp.post<API.LoginResp>({
    url: '/auth/oauth/login/byCode',
    method: 'POST',
    data,
  }, { withToken: false })
}

// Oauth 第三方登录，客户端获取换取手机号的code进行登录
export function oauthLoginByPhone(data: API.OauthLoginByPhoneCodeReq) {
  return defHttp.post<API.LoginResp>({
    url: '/auth/oauth/login/byPhone',
    method: 'POST',
    data,
  }, { withToken: false })
}

// 用户注册
export function register(data: API.RegisterReq) {
  return defHttp.post<API.BaseID>({
    url: '/auth/register',
    method: 'POST',
    data,
  }, { withToken: false })
}
