import {
  MenuModeEnum,
  NavBarModeEnum,
  SIDE_BAR_MINI_WIDTH,
  SIDE_BAR_SHOW_TIT_MINI_WIDTH,
  TriggerEnum,
} from '~/constants'

const mixSideHasChildren = ref(false)

export function useMenuSetting() {
  const { getFullContent: fullContent } = useFullContent()
  const appConfig = useAppConfig()
  const { getShowLogo } = useRootSetting()

  const getCollapsed = computed(() => appConfig.sidebar.value.collapsed)
  const getMenuType = computed(() => unref(appConfig.navBarMode))
  const getMenuMode = computed(() => appConfig.menu.value.mode)
  const getMenuFixed = computed(() => appConfig.sidebar.value.fixed)
  const getShowMenu = computed(() => appConfig.menu.value.show)
  const getMenuHidden = computed(() => !appConfig.sidebar.value.visible)
  const getMenuWidth = computed(() => appConfig.sidebar.value.width)
  const getTrigger = computed(() => appConfig.sidebar.value.trigger)
  const getMenuTheme = computed(() => appConfig.sidebar.value.theme)
  const getSplit = computed(() => appConfig.menu.value.split)
  const getMenuBgColor = computed(() => appConfig.sidebar.value.bgColor)
  const getMixSideTrigger = computed(() => appConfig.menu.value.mixSideTrigger)
  const getShowSidebar = computed(() => {
    return (
      unref(getSplit)
      || (unref(getShowMenu)
        && !unref(appConfig.isHorizontal)
        && !unref(fullContent))
    )
  })

  const getCanDrag = computed(() => appConfig.menu.value.canDrag)
  const getAccordion = computed(() => appConfig.menu.value.accordion)
  const getMixSideFixed = computed(() => appConfig.menu.value.mixSideFixed)
  const getTopMenuAlign = computed(() => appConfig.menu.value.topMenuAlign)
  const getCloseMixSidebarOnChange = computed(() => appConfig.closeMixSidebarOnChange.value)
  const getIsSidebarType = computed(() => unref(getMenuType) === NavBarModeEnum.SIDEBAR)
  const getIsTopMenu = computed(() => unref(getMenuType) === NavBarModeEnum.TOP_MENU)
  const getMenuShowLogo = computed(() => unref(getShowLogo) && unref(getIsSidebarType))
  const getCollapsedShowTitle = computed(() => appConfig.menu.value.collapsedShowTitle)
  const getShowTopMenu = computed(() => unref(getMenuMode) === MenuModeEnum.HORIZONTAL || unref(getSplit))

  const getShowHeaderTrigger = computed(() => {
    if (
      unref(getMenuType) === NavBarModeEnum.TOP_MENU
      || !unref(getShowMenu)
      || unref(getMenuHidden)
    ) {
      return false
    }

    return unref(getTrigger) === TriggerEnum.HEADER
  })

  const getShowCenterTrigger = computed(() => unref(getTrigger) === TriggerEnum.CENTER)
  const getShowFooterTrigger = computed(() => unref(getTrigger) === TriggerEnum.FOOTER)
  const getIsHorizontal = computed(() => unref(getMenuMode) === MenuModeEnum.HORIZONTAL)
  const getIsMixSidebar = computed(() => unref(getMenuType) === NavBarModeEnum.MIX_SIDEBAR)
  const getIsMixMode = computed(() => {
    return (
      unref(getMenuMode) === MenuModeEnum.INLINE
      && unref(getMenuType) === NavBarModeEnum.MIX
    )
  })

  const getMiniWidthNumber = computed(() => {
    const { collapsedShowTitle } = appConfig.menu.value
    return collapsedShowTitle
      ? SIDE_BAR_SHOW_TIT_MINI_WIDTH
      : SIDE_BAR_MINI_WIDTH
  })

  const getRealWidth = computed(() => {
    if (unref(getIsMixSidebar)) {
      return unref(getCollapsed) && !unref(getMixSideFixed)
        ? unref(getMiniWidthNumber)
        : unref(getMenuWidth)
    }
    return unref(getCollapsed) ? unref(getMiniWidthNumber) : unref(getMenuWidth)
  })

  const getCalcContentWidth = computed(() => {
    const width
      = unref(getIsTopMenu)
        || !unref(getShowMenu)
        || (unref(getSplit) && unref(getMenuHidden))
        ? 0
        : unref(getIsMixSidebar)
          ? (unref(getCollapsed)
              ? SIDE_BAR_MINI_WIDTH
              : SIDE_BAR_SHOW_TIT_MINI_WIDTH)
          + (unref(getMixSideFixed) && unref(mixSideHasChildren)
            ? unref(getRealWidth)
            : 0)
          : unref(getRealWidth)

    return `calc(100% - ${unref(width)}px)`
  })

  const getIsFixed = computed(() => {
    return unref(getMixSideFixed) && unref(mixSideHasChildren)
  })

  function setMenuSetting(menuSetting: Partial<MenuSetting>): void {
    appConfig.setAppConfig({ menu: menuSetting })
  }

  function setSidebarSetting(
    sidebarSetting: Partial<SidebarConfigOptions>,
  ): void {
    appConfig.setAppConfig({ sidebar: sidebarSetting })
  }

  function setSiderWidth(width: number) {
    setSidebarSetting({ width })
  }

  function toggleCollapsed() {
    setSidebarSetting({
      collapsed: !unref(getCollapsed),
    })
  }

  return {
    setSiderWidth,
    setMenuSetting,
    toggleCollapsed,
    getMenuFixed,
    getRealWidth,
    getMenuType,
    getMenuMode,
    getShowMenu,
    getCollapsed,
    getMiniWidthNumber,
    getCalcContentWidth,
    getMenuWidth,
    getTrigger,
    getSplit,
    getMenuTheme,
    getCanDrag,
    getCollapsedShowTitle,
    getIsHorizontal,
    getIsSidebarType,
    getAccordion,
    getShowTopMenu,
    getShowHeaderTrigger,
    getShowCenterTrigger,
    getShowFooterTrigger,
    getTopMenuAlign,
    getMenuHidden,
    getIsTopMenu,
    getMenuBgColor,
    getShowSidebar,
    getIsMixMode,
    getIsMixSidebar,
    getCloseMixSidebarOnChange,
    getMixSideTrigger,
    getMixSideFixed,
    getIsFixed,
    mixSideHasChildren,
    getMenuShowLogo,
  }
}
