import type { Locale } from 'vue-i18n'

export const LOCALE: { [key: string]: Locale } = {
  ZH_CHS: 'zh-chs', // 简体中文
  ZH_CHT: 'zh-cht', // 繁体中文
  AR: 'ar', // 阿拉伯语
  BG: 'bg', // 保加利亚语
  HR: 'hr', // 克罗地亚语
  CS: 'cs', // 捷克语
  DA: 'da', // 丹麦语言
  DE: 'de', // 德语
  EL: 'el', // 希腊语
  EN: 'en', // 英语
  ET: 'et', // 爱沙尼亚语
  ES: 'es', // 西班牙语
  FI: 'fi', // 芬兰语
  FR: 'fr', // 法语
  GA: 'ga', // 爱尔兰语
  HI: 'hi', // 印地语
  HU: 'hu', // 匈牙利语
  HE: 'he', // 希伯来语
  IT: 'it', // 意大利语
  JA: 'ja', // 日语
  KO: 'ko', // 朝鲜语
  LV: 'lv', // 拉脱维亚语
  LT: 'lt', // 立陶宛语
  NL: 'nl', // 荷兰语
  NO: 'no', // 挪威语
  PL: 'pl', // 波兰语
  PT: 'pt', // 葡萄牙语
  SV: 'sv', // 瑞典语
  RO: 'ro', // 罗马尼亚语
  RU: 'ru', // 俄语
  SR_CS: 'sr-cs', // 塞尔维亚语
  SK: 'sk', // 斯洛伐克语
  SL: 'sl', // 斯洛文尼亚语
  TH: 'th', // 泰语
  TR: 'tr', // 土耳其语
  UK_UA: 'uk-ua', // 乌克兰语
}

export const localesMap = Object.fromEntries(
  Object.entries(import.meta.glob('../../../locales/*.yml'))
    .map(([path, loadLocale]) => [path.match(/([\w-]*)\.yml$/)?.[1], loadLocale]),
) as Record<Locale, () => Promise<{ default: Record<string, string> }>>

export const localeSetting: LocaleConfig = {
  locale: LOCALE.ZH_CHS,
  fallback: LOCALE.ZH_CHS,
  availableLocales: Object.keys(localesMap),
}

export const localeList: any[] = [
  {
    text: '简体中文',
    event: LOCALE.ZH_CHS,
    icon: 'emojione:flag-for-china',
  },
  // {
  //   text: '繁体中文',
  //   event: LOCALE.ZH_CHT,
  //   icon: 'emojione:flag-for-hong-kong-sar-china',
  // },
  {
    text: 'English',
    event: LOCALE.EN,
    icon: 'emojione:flag-for-united-states',
  },
  {
    text: '日本語',
    event: LOCALE.JA,
    icon: 'emojione:flag-for-japan',
  },
]
