import type { App } from 'vue'
import PrimeVue from 'primevue/config'
// @ts-expect-error mjs
import PrimeOne from 'primevue/themes/primeone'
// @ts-expect-error mjs
import Aura from 'primevue/themes/primeone/aura'

import ConfirmationService from 'primevue/confirmationservice'
import DialogService from 'primevue/dialogservice'
import ToastService from 'primevue/toastservice'

// directive
import Tooltip from 'primevue/tooltip'
import Badge from 'primevue/badgedirective'
import Ripple from 'primevue/ripple'
import Styleclass from 'primevue/styleclass'
import Focustrap from 'primevue/focustrap'
import Animateonscroll from 'primevue/animateonscroll'

// locale
import zhCN from 'primelocale/zh-CN.json'

export function setupPrimeVue(app: App<Element>) {
  app.use(PrimeVue, {
    ripple: true,
    theme: {
      base: PrimeOne,
      preset: Aura,
      options: {
        prefix: 'p',
        darkModeSelector: '.dark',
      },
    },
    locale: Object.assign({ fileSizeType: '文件大小类型' }, zhCN['zh-CN']),
  })
  app.use(ConfirmationService)
  app.use(DialogService)
  app.use(ToastService)

  // directive
  app.directive('tooltip', Tooltip)
  app.directive('badge', Badge)
  app.directive('ripple', Ripple)
  app.directive('styleclass', Styleclass)
  app.directive('focustrap', Focustrap)
  app.directive('animateonscroll', Animateonscroll)
}
