import NProgress from 'nprogress'
import type { Router } from 'vue-router/auto'

const LOADED_PAGE_POOL = new Map<string, boolean>()

export function createNProgressGuard(router: Router) {
  const openNProgress = useAppConfigStore().transition.openNProgress
  router.beforeEach((to) => {
    // The page has already been loaded, it will be faster to open it again, you don’t need to do loading and other processing
    to.meta.loaded = !!LOADED_PAGE_POOL.get(to.path)
    // Display a progress bar at the top when switching pages
    // Only works when the page is loaded for the first time
    if (openNProgress && !to.meta.loaded) {
      NProgress.start()
    }
    return true
  })
  router.afterEach((to) => {
    // Indicates that the page has been loaded
    // When opening again, you can turn off some progress display interactions
    LOADED_PAGE_POOL.set(to.path, true)
    // Close the page loading progress bar
    if (openNProgress && !to.meta.loaded) {
      NProgress.done()
    }
  })
}
