import type { RouteRecordRaw, Router } from 'vue-router/auto'
import { createRouter, createWebHistory } from 'vue-router/auto'
import { setupLayouts } from 'virtual:generated-layouts'
import { createNProgressGuard } from './nprogress'
import { createTabsGuard } from './tabs'
import { setRouteChange } from '~/utils'

// eslint-disable-next-line import/no-mutable-exports
export let router: Router

export function initRouter(path: string): Router {
  const routeStore = useRouteStore()

  router = createRouter({
    history: createWebHistory(path),
    strict: false,
    scrollBehavior: () => ({ left: 0, top: 0 }),
    extendRoutes: (routes: RouteRecordRaw[]) => {
      const initialRoutes = routeStore.initRoutes(routes)
      // console.log('🚀 ~ initRouter ~ setupLayouts(initialRoutes):', initialRoutes, setupLayouts(initialRoutes))
      return setupLayouts(initialRoutes)
    },
  })

  return router
}

export function setupRouterGuards() {
  createNProgressGuard(router)
  createTabsGuard(router, setRouteChange)
  // 暂时的
  // useMenuStore()
}
