import { acceptHMRUpdate, defineStore } from 'pinia'
import {
  MenuModeEnum,
  NavBarModeEnum,
} from '~/constants'
import { projectSetting } from '~/setting'
import { _assign, _merge } from '~/utils'

export const useAppConfigStore = defineStore('APP_CONFIG', () => {
  let state = $ref<DefineAppConfigOptions>(Object.assign({}, projectSetting))

  // computed
  const isSidebar = computed(() => state.navBarMode === NavBarModeEnum.SIDEBAR)
  const isTopMenu = computed(() => state.navBarMode === NavBarModeEnum.TOP_MENU)
  const isMixSidebar = computed(() => state.navBarMode === NavBarModeEnum.MIX_SIDEBAR)
  const isMix = computed(() => state.navBarMode === NavBarModeEnum.MIX)
  const isMixMode = computed(() =>
    state.navBarMode === NavBarModeEnum.MIX
  && state.menu.mode === MenuModeEnum.INLINE,
  )
  const isHorizontal = computed(() => state.menu.mode === MenuModeEnum.HORIZONTAL)
  const tagTarCache = computed(() => state.tabTar.cache)

  function setAppConfig(configs: DeepPartial<DefineAppConfigOptions>) {
    state = _merge(state, configs)
  }

  function setSidebar(value: Partial<Omit<SidebarConfigOptions, 'mixSidebarWidth' | 'collapsedWidth'>>) {
    _assign(state.sidebar, value)
  }

  function setMenu(value: Partial<MenuConfigOptions>) {
    _assign(state.menu, value)
  }

  function setHeader(value: Partial<Omit<HeaderConfigOptions, 'height'>>) {
    _assign(state.header, value)
  }

  function setLogo(value: Partial<LogoConfigOptions>) {
    _assign(state.logo, value)
  }

  function setTabBar(value: Partial<TabTarConfigOptions>) {
    _assign(state.tabTar, value)
  }

  function setContent(value: Partial<ContentConfigOptions>) {
    _assign(state.content, value)
  }

  function setFooter(value: Partial<Omit<FooterConfigOptions, 'height'>>) {
    _assign(state.footer, value)
  }

  function setTransition(value: Partial<TransitionConfigOptions>) {
    _assign(state.transition, value)
  }

  return {
    ...toRefs(state),
    isSidebar,
    isTopMenu,
    isMixSidebar,
    isMix,
    isMixMode,
    isHorizontal,
    tagTarCache,
    // setter
    setAppConfig,
    setSidebar,
    setMenu,
    setHeader,
    setLogo,
    setTabBar,
    setContent,
    setFooter,
    setTransition,
  }
}, {
  persist: {
    paths: Object.keys(projectSetting),
  },
})

if (import.meta.hot)
  import.meta.hot.accept(acceptHMRUpdate(useAppConfigStore as any, import.meta.hot))
