export declare global {
  /**
   * 排序专用Fn
   */
  declare type SortFn = () => number

  /**
   * 字符串类型对象
   */
  declare type Recordable<T = any> = Record<string, T>

  /**
   * 字符串类型对象（只读）
   */
  declare interface ReadonlyRecordable<T = any> {
    readonly [key: string]: T
  }

  /**
   *  T | null 包装
   */
  declare type Nullable<T> = T | null

  /**
   * T | Not null 包装
   */
  declare type NonNullable<T> = T extends null | undefined ? never : T

  declare interface Indexable<T = any> {
    [key: string]: T
  }

  declare type DeepPartial<T> = {
    [P in keyof T]?: DeepPartial<T[P]>;
  }

  declare type Lazy<T> = () => Promise<T>

  /**
   * 任意类型的异步函数
   */
  declare type PromiseFunction<T> = (...arg: any[]) => PromiseLike<T>

  /**
   * 任意类型的普通函数
   */
  declare type NormalFunction<T> = (...arg: any[]) => T

  /**
   * 任意类型的函数
   */
  declare type AnyFunction<T> = NormalFunction<T> | PromiseFunction<T>

  /**
   * setTimeout 返回值类型
   */
  declare type TimeoutHandle = ReturnType<typeof setTimeout>

  /**
   * setInterval 返回值类型
   */
  declare type IntervalHandle = ReturnType<typeof setInterval>

  declare interface ComponentElRef<T extends HTMLElement = HTMLDivElement> {
    $el: T
  }

  declare type ComponentRef<T extends HTMLElement = HTMLDivElement> = ComponentElRef<T> | null

  declare type ElRef<T extends HTMLElement = HTMLDivElement> = Nullable<T>

  declare type FormType = 'create' | 'update'
}
